@@ -18,7 +18,9 @@
 
 set -e
 
+cd src/$pkg
+
 # The -count=1 parameter is the "idiomatic way to bypass test caching".
 # https://golang.org/doc/go1.10#test
 # The -v option enables verbose output.
-$GOLANG test -count=1 -v $pkg
+$GOLANG test -count=1 -v
